/*
 * This file is a part of PikoTools
 * and is distributed under the (new) BSD licence.
 * Author: Tomasz Sowa <t.sowa@ttmath.org>
 */

/*
 * Copyright (c) 2021, Tomasz Sowa
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 *  * Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 *
 *  * Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 *  * Neither the name Tomasz Sowa nor the names of contributors to this
 *    project may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF
 * THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <cstdlib>
#include <cwchar>
#include "double.h"
#include "textstream/textstream.h"



namespace pt
{


float to_float(const char * str, const char ** after)
{
	char * after_local;
	float res = std::strtof(str, &after_local);

	if( after )
	{
		*after = after_local;
	}

	return res;
}


float to_float(const wchar_t * str, const wchar_t ** after)
{
	wchar_t * after_local;
	float res = std::wcstof(str, &after_local);

	if( after )
	{
		*after = after_local;
	}

	return res;
}


double to_double(const char * str, const char ** after)
{
	char * after_local;
	double res = std::strtod(str, &after_local);

	if( after )
	{
		*after = after_local;
	}

	return res;
}


double to_double(const wchar_t * str, const wchar_t ** after)
{
	wchar_t * after_local;
	double res = std::wcstod(str, &after_local);

	if( after )
	{
		*after = after_local;
	}

	return res;
}


long double to_long_double(const char * str, const char ** after)
{
	char * after_local;
	long double res = std::strtold(str, &after_local);

	if( after )
	{
		*after = after_local;
	}

	return res;
}


long double to_long_double(const wchar_t * str, const wchar_t ** after)
{
	wchar_t * after_local;
	long double res = std::wcstold(str, &after_local);

	if( after )
	{
		*after = after_local;
	}

	return res;
}


float to_float(const std::string & str, const char ** after)
{
	return to_float(str.c_str(), after);
}


float to_float(const std::wstring & str, const wchar_t ** after)
{
	return to_float(str.c_str(), after);
}


double to_double(const std::string & str, const char ** after)
{
	return to_double(str.c_str(), after);
}

double to_double(const std::wstring & str, const wchar_t ** after)
{
	return to_double(str.c_str(), after);
}


long double to_long_double(const std::string & str, const char ** after)
{
	return to_long_double(str.c_str(), after);
}


long double to_long_double(const std::wstring & str, const wchar_t ** after)
{
	return to_long_double(str.c_str(), after);
}






std::string to_str(float val)
{
	TextStream str;
	str << val;
	return str.to_str();;
}


std::wstring to_wstr(float val)
{
	TextStream str;
	str << val;
	return str.to_wstr();;
}


std::string to_str(double val)
{
	TextStream str;
	str << val;
	return str.to_str();;
}


std::wstring to_wstr(double val)
{
	TextStream str;
	str << val;
	return str.to_wstr();;
}


std::string to_str(long double val)
{
	TextStream str;
	str << val;
	return str.to_str();;
}


std::wstring to_wstr(long double val)
{
	TextStream str;
	str << val;
	return str.to_wstr();;
}






}

