/*
 * This file is a part of Winix
 * and is distributed under the 2-Clause BSD licence.
 * Author: Tomasz Sowa <t.sowa@ttmath.org>
 */

/*
 * Copyright (c) 2018-2021, Tomasz Sowa
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 */

#include "filelog.h"
#include <ctime>
#include <string.h>
#include "utf8/utf8.h"



namespace pt
{



FileLog::FileLog()
{
	log_level = 4;
	log_stdout = false;
	log_file_open = false;
	save_each_line = false;
}


FileLog::~FileLog()
{
}


bool FileLog::synchro_lock()
{
	return true;
}


void FileLog::synchro_unlock()
{
}


void FileLog::init(const std::wstring & log_file, bool log_stdout, int log_level, bool save_each_line)
{
	this->log_stdout = log_stdout;
	this->log_level = log_level;
	this->save_each_line = save_each_line;
	wide_to_utf8(log_file, this->log_file);
}


int FileLog::get_log_level()
{
	return log_level;
}


bool FileLog::should_save_each_line()
{
	return save_each_line;
}


void FileLog::open_file()
{
	if( !log_file.empty() )
	{
		file.open( log_file.c_str(), std::ios_base::out | std::ios_base::app );
		log_file_open = true;
	}
}


void FileLog::save_log(WTextStream * buffer)
{
	if( buffer->empty() )
		return;

	if( synchro_lock() )
	{
		try
		{
			if( log_stdout )
			{
				wide_stream_to_utf8(*buffer, std::cout);
			}

			if( !log_file.empty() )
			{
				if( !log_file_open || !file )
				{
					file.close();
					file.clear();

					open_file();
				}

				if( file )
				{
					wide_stream_to_utf8(*buffer, file);
					file.flush();
				}
			}
		}
		catch(...)
		{
		}
	}

	synchro_unlock();
}





} // namespace

